/****************************************************************
 * f_file.c
 * Last change 12-23-2013 by Osman Celimli
 ****************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <stdio.h>
#include <string.h>
#include <windows.h>

#include "useful.h"
#include "options.h"

int procPak(char *pszScript);
/****************************************************************
 * Important Globals
 ****************************************************************/
typedef struct includeFile
{
	char *pszName;
	char *pszScript;
} includeFile;

includeFile *paIncludes;
unsigned int uiIncBufferLen = 0;
unsigned int uiIncDepth = 0;
/****************************************************************
 * Text Tables
 ****************************************************************/
char ERR_NoFileMem[]= "ERROR- Out of memory\n";
char ERR_IncludeEnd[]= "ERROR- Unexpected end of source data in : ";

char ERR_NoInclude[]= "ERROR- Can't open include file : ";
char ERR_FileLoop[]= "ERROR- Source file loopback in : ";
/****************************************************************
 * int f_isOpen(char *pszFile)
 * Check to see if the given file is already open.
 ****************************************************************/
int f_isOpen(char *pszFile)
{
	unsigned int uiX;

	if((paIncludes == NULL) || (uiIncDepth == 0)) return 0;
	for(uiX = 0; uiX < uiIncDepth; uiX++)
	{
		if(strcmp(pszFile, paIncludes[uiX].pszName) == 0)
		{
			return -1;
		}
	}
	return 0;
}
/****************************************************************
 * int f_openScript(char *pszFile)
 * Attempt to open the supplied file and then convert it into a
 * command script, feeding the results into procPak().
 ****************************************************************/
int f_openScript(char *pszFile)
{
	FILE *pScriptFile;
	unsigned int uiScriptLen;
	unsigned int uiX,uiY,uiZ;
	int iRet;

	/**
	 * Ensure that the file isn't already being included and that
	 * our include list is ready for use before proceeding.
	 */
	if(f_isOpen(pszFile))
	{
		fprintf(stderr, "%s%s", ERR_FileLoop, pszFile);
		return -1;
	}
	if(uiIncDepth == 0)
	{
		uiIncBufferLen = sizeof(includeFile) * 8;
		paIncludes = malloc(uiIncBufferLen);
		if(paIncludes == NULL)
		{
			fprintf(stderr, "%s", ERR_NoFileMem);
			return -1;
		}
	}

	/**
	 * Attempt to bump the size of the include list if needed, then
	 * open the requested file and copy in its data.
	 */
	uiY = uiIncDepth;
	uiX = (uiIncDepth + 1) * sizeof(includeFile);
	if(tealloc(&paIncludes, &uiIncBufferLen, uiX))
	{
		fprintf(stderr, "%s", ERR_NoFileMem);
		return -1;
	}

	paIncludes[uiY].pszName = NULL;
	paIncludes[uiY].pszScript = NULL;
	uiIncDepth++;

	uiZ = strlen(pszFile) + 1;
	if((paIncludes[uiY].pszName = malloc(uiZ)) == NULL)
	{
		fprintf(stderr, "%s", ERR_NoFileMem);
		return -1;
	}
	strcpy(paIncludes[uiY].pszName, pszFile);
	if(NULL == (pScriptFile = fopen(pszFile, "rb")))
	{
		fprintf(stderr,"%s%s\n", ERR_NoInclude, pszFile);
		return -1;
	}
	fseek(pScriptFile, 0, SEEK_END);
	uiScriptLen = ftell(pScriptFile);
	rewind(pScriptFile);

	if((paIncludes[uiY].pszScript = malloc(uiScriptLen + 1)) == NULL)
	{
		fprintf(stderr, "%s", ERR_NoFileMem);
		return -1;
	}
	if(uiScriptLen != fread(paIncludes[uiY].pszScript, sizeof(unsigned char), uiScriptLen, pScriptFile))
	{
		fprintf(stderr, "%s%s\n", ERR_IncludeEnd, pszFile);
		fclose(pScriptFile);
		return -1;
	}
	paIncludes[uiY].pszScript[uiScriptLen] = '\0';
	fclose(pScriptFile);

	/**
	 * We can now strip out all non-essential whitespace and
	 * feed the script back into procPak. Its name, data,
	 * and entry in the include list will be released once
	 * we get back here.
	 */
	strcut(paIncludes[uiY].pszScript, " \t\r\n", ' ');
	iRet = procPak(paIncludes[uiY].pszScript);

	free(paIncludes[uiY].pszName);
	free(paIncludes[uiY].pszScript);
	uiIncDepth--;
	return iRet;
}
/****************************************************************
 * void f_closeTree(void)
 * Close all files in the current inclusion hierarchy.
 ****************************************************************/
void f_closeTree(void)
{
	unsigned int uiX;

	for(uiX = 0; uiX < uiIncDepth; uiX++)
	{
		if(paIncludes[uiX].pszName != NULL)
			free(paIncludes[uiX].pszName);
		if(paIncludes[uiX].pszScript != NULL)
			free(paIncludes[uiX].pszScript);
	}
	if(paIncludes != NULL) free(paIncludes);
}
