/****************************************************************
 * usefuls.c
 * Last change 12-23-2013 by Osman Celimli
 ****************************************************************/
#define _CRTDBG_MAP_ALLOC
#include <stdlib.h>
#include <crtdbg.h>

#include <stdio.h>
#include <string.h>
#include <windows.h>
/****************************************************************
 * Text Tables
 ****************************************************************/
char ERR_Usefuls_Null[]= "ERROR- Unexpected NULL.\n";
char ERR_Usefuls_EarlyEnd[]= "ERROR- Unexpected end of source data.\n";
char ERR_Usefuls_BadBinStr[]= "ERROR- Invalid binary value : ";
char ERR_Usefuls_BadNumStr[]= "ERROR- Unable to parse numeric : ";
/****************************************************************
 * int tealloc(void **pCurAddr, size_t *pCurLen, size_t reqLen)
 * Attempt to reallocate the given block of memory whose base
 * address is stored at *pCurAddr if reqLen > *pCurLen. The pointer
 * and block size will automatically be updated if the reallocation
 * occurs and is successful. Returns nonzero on any failure.
 ****************************************************************/
int tealloc(void **pCurAddr, size_t *pCurLen, size_t reqLen)
{
	void *pNewAddr;
	size_t newLen;

	if((pCurAddr == NULL) || (pCurLen == NULL)) return -1;

	if(*pCurLen < reqLen)
	{
		newLen = ((*pCurLen << 1) > reqLen) ? (*pCurLen << 1) : reqLen;

		pNewAddr = realloc(*pCurAddr, newLen);
		if(pNewAddr == NULL) return -1;

		*pCurLen = newLen;
		*pCurAddr = pNewAddr;
	}
	return 0;
}
/****************************************************************
 * int strcut(char *pszTarget, char *pszRem, char cRep)
 * Find groups of one of more members of pszRem in the string
 * pszTarget and replace them with a single character cRep.
 ****************************************************************/
int strcut(char *pszTarget, char *pszRem, char cRep)
{
	char cCur;
	int iReplace;
	unsigned int uiL,uiR;

	if((pszTarget == NULL) || (pszRem == NULL)) return -1;
	iReplace = 0;
	uiL = 0;
	uiR = 0;

	while((cCur = pszTarget[uiR++]) != '\0')
	{
		if(strchr(pszRem, cCur) == NULL)
		{
			pszTarget[uiL++] = cCur;
			iReplace = -1;
		}
		else if(iReplace)
		{
			pszTarget[uiL++] = cRep;
			iReplace = 0;
		}
	}

	pszTarget[uiL] = '\0';
	if((uiL > 0) && (pszTarget[uiL - 1] == cRep))
	{
		pszTarget[uiL - 1] = '\0';
	}
	return 0;
}
/****************************************************************
 * int strtoint(char *pszString, int *piTarget)
 * Parse a string containing a value in either $hexadecimal,
 * %binary, or decimal. piTarget will not be overwritten if
 * any errors occur during the conversion process.
 ****************************************************************/
int strtoint(char *pszString, int *piTarget)
{
	int iSuccess, iVal;
	unsigned int uiX;

	if((pszString == NULL) || (piTarget == NULL))
	{
		fprintf(stderr, "%s", ERR_Usefuls_Null);
		return -1;
	}

	/**
	 * Empty
	 */
	if(pszString[0] == '\0')
	{
		fprintf(stderr, "%s", ERR_Usefuls_EarlyEnd);
		return -1;
	}
	/**
	 * $ Hexadecimal
	 */
	else if(pszString[0] == '$') iSuccess = sscanf((pszString + 1), "%x", &iVal);
	/**
	 * % Binary
	 */
	else if(pszString[0] == '%')
	{
		iVal = 0;
		uiX = 1;

		while((pszString[uiX] != '\0') && (uiX < 33))
		{
			iVal = iVal << 1;

			if(pszString[uiX] == '1') iVal |= 1;
			else if(pszString[uiX] != '0')
			{
				fprintf(stderr, "%s%s", ERR_Usefuls_BadBinStr, pszString);
				return -1;
			}
			uiX++;
		}
		if(uiX == 1)
		{
			fprintf(stderr, "%s%s", ERR_Usefuls_BadBinStr, pszString);
			return -1;
		}
		iSuccess = 1;
	}
	/**
	 * Decimal
	 */
	else iSuccess = sscanf(pszString, "%d", &iVal);

	if((iSuccess != 0) && (iSuccess != EOF))
	{
		*piTarget = iVal;
	}
	else
	{
		fprintf(stderr, "%s%s", ERR_Usefuls_BadNumStr, pszString);
		return -1;
	}
	return 0;
}
